#!/usr/bin/python
# - * - coding: utf-8 - * -

from kivy.app import App
from kivy.uix.widget import Widget
from kivy.properties import NumericProperty, ReferenceListProperty,\
ObjectProperty
from kivy.clock import Clock
from kivy.core.window import Window
from kivy.core.audio import SoundLoader
from kivy.uix.screenmanager import ScreenManager, Screen, FadeTransition
from kivy.uix.button import Button
from kivy.properties import BooleanProperty
from kivy.uix.label import Label
from kivy.uix.textinput import TextInput
from functools import partial

from practicum import *
from peri import *
from usb.core import USBError

import random, os, json

def randKeyboard():
    global KEYBOARD
    keycode = 1 << int(random.random() * 6)
    if keycode == 1:
        KEYBOARD = "cm"
    elif keycode == 2:
        KEYBOARD = "xn"
    elif keycode == 4:
        KEYBOARD = "zb"
    elif keycode == 8:
        KEYBOARD = "dj"
    elif keycode == 16:
        KEYBOARD = "sh"
    elif keycode == 32:
        KEYBOARD = "ag"
    return keycode
    
def loadMusicList(fileName):
    with open(fileName) as jsonFile:
        return json.load(jsonFile)

def getMusic(artist, song):
    return artist + " - " + song + ".mp3"

PRESS_GOOD = 300
PRESS_BAD = -200

STATUS_SHIELD = 1
STATUS_MULTIPLY = 2
STATIS_INVERSE = 3

KEYBOARD = "  "

KEYBOARD_1 = ['a', 's', 'd', 'z', 'x', 'c']
KEYBOARD_2 = ['b', 'n', 'm', 'g', 'h', 'j']


SHIELD_TIME = 8
INVERSE_TIME = 5
FACTOR_TIME = 10

TIME = 20
BPM = "100.02"
DELAY = 4
GAME = True

musicPath = os.getcwd() + "/musicList.json"
jsonFile = dict()

class BattleOfLightApp(App):
    def run(self):
        super(BattleOfLightApp, self).run()

    def build(self):
        Screen = ScreenManager(transition=FadeTransition())
        Screen.add_widget(OpenScreen(Screen, name="open"))
        Screen.add_widget(GameScreen(name="game"))
        return Screen

        """
        game = TheGame()
        game.setTime(self.time)
        game.setBPM(self.bpm)
        Clock.schedule_interval(game.update, 60.0/self.bpm)
        #Clock.schedule_interval(game.update, 2)
        Clock.schedule_interval(game.updateTime, 1.0)
        return game
        """

class OpenScreen(Screen):
    global BPM
    jsonFile = loadMusicList(os.getcwd() + "/musicList.json")
    openSong = jsonFile["Open"]
    music = SoundLoader.load("./music/" + openSong["Artist"] + " - " + openSong["Song"] + ".mp3")
    i = 0

    def __init__(self, sm, **kwargs):
        super(OpenScreen, self).__init__(**kwargs)
        self.sm = sm
        self.add_widget(Label(text="Battle Of Light", font_size='80sp', pos=(self.center_x, self.top + 100)))
        self.add_widget(Label(text="Please select song: ", font_size='15sp', pos=(self.center_x, self.top - 100)))
        for bpm, value in jsonFile.iteritems():
            if bpm == "Open":
                continue
            button = Button(text="BPM: %s\nArtist: %s\nSong: %s" % (bpm, value["Artist"], value["Song"]), font_size="10sp", pos=(self.width + self.i, self.height), size=(100, 100), size_hint=(0.13, 0.1))
            button.bind(on_press = partial(self.startGame, bpm, value["length"]))
            self.add_widget(button)
            self.i += 180

    def on_pre_enter(self):
        self.music.play()

    def startGame(self, bpm, time, objected):
        global BPM, TIME
        BPM = bpm
        TIME = int(time)
        self.music.unload
        self.music.stop()
        self.parent.current = "game"


class GameScreen(Screen):
    def on_pre_enter(self):
        global TIME
        jsonFile = loadMusicList(os.getcwd() + "/musicList.json")
        songDict = jsonFile[str(BPM)]
        artist = songDict["Artist"]
        song = songDict["Song"]
        TIME = int(songDict["length"])
        self.bolgame.music = SoundLoader.load('./music/' + artist + ' - ' + song + '.mp3')
        self.bolgame.time = int(songDict["length"])
        self.bolgame.bpm = float(BPM)
        self.bolgame.player_1.shieldTime = TIME
        self.bolgame.player_1.inverseTime = TIME
        self.bolgame.player_1.factorScoreTime = TIME
        self.bolgame.player_2.shieldTime = TIME
        self.bolgame.player_2.inverseTime = TIME
        self.bolgame.player_2.factorScoreTime = TIME
        self.bolgame.music.play()
        Clock.schedule_interval(self.bolgame.update, 60.0/float(BPM))
        Clock.schedule_interval(self.bolgame.updateTime, 1.0)

# class MyButton(Button):
#     enabled = BooleanProperty(True)
# 
#     def on_enabled(self, instance, value):
#         if value:
#             self.background_color = [1,1,1,1]
#             self.color = [1,1,1,1]
#         else:
#             self.background_color = [1,1,1,.3]
#             self.color = [1,1,1,.5]
# 
#     def on_touch_down( self, touch ):
#         if self.enabled:
#             return super(self.__class__, self).on_touch_down(touch)

class BattleOfLight(Widget):
    # player_1 = ObjectProperty(TIME)
    # player_2 = ObjectProperty(TIME)
    time = NumericProperty(TIME)
    bpm = NumericProperty(BPM)
    GAME = True

    def __init__(self, **kwargs):
        super(BattleOfLight, self).__init__(**kwargs)
        self._keyboard = Window.request_keyboard(self._keyboard_closed, self)
        self._keyboard.bind(on_key_down=self._on_keyboard_down)

    def _keyboard_closed(self):
        self._keyboard.unbind(on_key_down=self._on_keyboard_down)
        self._keyboard = None

    def _on_keyboard_down(self, keyboard, keycode, text, modifiers):
        global KEYBOARD, SHIELD_TIME, INVERSE_TIME, FACTOR_TIME
        if self.time <= 0:
            return
        if KEYBOARD[0] == keycode[1]:
            self.player_1.updateScore(PRESS_GOOD * (1.045 ** self.player_1.combo))
            self.player_1.combo += 1
        elif keycode[1] in KEYBOARD_1:
            self.player_1.updateScore(PRESS_BAD)
            self.player_1.combo = 0
        elif keycode[1] == 'q':
            if self.player_1.items & 256 == 256:
                self.player_1.shieldTime = self.time - SHIELD_TIME
            self.player_1.items &= ~(1 << 8)
        elif keycode[1] == 'w':
            if self.player_1.items & 128 == 128:
                self.player_2.inverse = 1
                self.player_2.inverseTime = self.time - INVERSE_TIME
            self.player_1.items &= ~(1 << 7)
        elif keycode[1] == 'e':
            if self.player_1.items & 64 == 64:
                self.player_1.factorScore = 2
                self.player_1.factorScoreTime = self.time - FACTOR_TIME
            self.player_1.items &= ~(1 << 6)
        
        if KEYBOARD[1] == keycode[1]:
            self.player_2.updateScore(PRESS_GOOD * (1.045 ** self.player_2.combo))
            self.player_2.combo += 1
        elif keycode[1] in KEYBOARD_2:
            self.player_2.updateScore(PRESS_BAD)
            self.player_2.combo = 0
        elif keycode[1] == 't':
            if self.player_2.items & 256 == 256:
                self.player_2.shieldTime = self.time - SHIELD_TIME
            self.player_2.items &= ~(1 << 8)
        elif keycode[1] == 'y':
            if self.player_2.items & 128 == 128:
                self.player_1.inverse = 1
                self.player_1.inverseTime = self.time - INVERSE_TIME
            self.player_2.items &= ~(1 << 7)
        elif keycode[1] == 'u':
            if self.player_2.items & 64 == 64:
                self.player_2.factorScore = 2
                self.player_2.factorScoreTime = self.time - FACTOR_TIME
            self.player_2.items &= ~(1 << 6)
        return True

    def update(self, dt):
        # print "TIMMME: ",
        if not self.GAME:
            return
        if self.time <= 0:
            try:
                boards_1.usbWrite(0, index=0, value=0)
                boards_2.usbWrite(0, index=0, value=0)
            except USBError:
                print "USBError detected"
            self.music.unload
            self.music.stop()
            self.GAME = False
        if self.time >= TIME - (60.0 / self.bpm ) * DELAY:
            # print self.time,
            # print TIME - (60 / self.bpm) * 3000
            return
        if self.player_1.factorScoreTime >= self.time:
            # print "USE! FACTOR"
            self.player_1.factorScore = 1
            self.player_1.factorScoreTime = 0
        if self.player_1.inverseTime >= self.time or self.player_1.shieldTime <= self.time:
            # print "USE! INVERSE"
            self.player_1.inverse = 0
            self.player_1.inverseTime = 0

        if self.player_2.factorScoreTime >= self.time:
            # print "USE! FACTOR"
            self.player_2.factorScore = 1
            self.player_2.factorScoreTime = 0
        if self.player_2.inverseTime >= self.time or self.player_2.shieldTime <= self.time:
            self.player_2.inverse = 0
            self.player_2.inverseTime = 0
        # print self.time, self.player_1.shieldTime, self.player_2.shieldTime

        generateLight = randKeyboard()
        # print "%d %d %d %d" % (self.player_1.items, self.player_1.factorScoreTime, self.player_1.inverseTime, self.player_1.shieldTime)
        # print "%d %d %d %d" % (self.player_2.items, self.player_2.factorScoreTime, self.player_2.inverseTime, self.player_2.shieldTime)
        # print self.time
        try:
            # pass
            boards_1.usbWrite(0, index=0, value=(((generateLight ^ 63) | self.player_2.items) if self.player_2.inverse == 1 else (generateLight | self.player_2.items)))
            boards_2.usbWrite(0, index=0, value=(((generateLight ^ 63) | self.player_1.items) if self.player_1.inverse == 1 else (generateLight | self.player_1.items)))
        except USBError:
            print "USBError detected"

    def updateTime(self, dt):
        if self.time > 0:
            self.time -= 1

class Player(Widget):
    name = ObjectProperty(None)
    score = NumericProperty(0)
    combo = NumericProperty(0)
    status = NumericProperty(0)
    items = NumericProperty(0)
    shieldTime = NumericProperty(TIME)
    inverse = NumericProperty(0)
    inverseTime = NumericProperty(TIME)
    factorScore = NumericProperty(1)
    factorScoreTime = NumericProperty(TIME)

    def updateScore(self, score):
        if self.combo % 10 == 9:
            self.items |= (1 << int(random.random() * 3)) << 6
        self.score += score



if __name__ == "__main__":
    jsonFile = loadMusicList(musicPath)
    devices = findDevices()
    boards_1 = McuBoard(devices[0])
    boards_2 = McuBoard(devices[1])
    boards_1.usbWrite(0, index=0, value=0)
    boards_2.usbWrite(0, index=0, value=0)
    Window.size = (1280, 760)
    BattleOfLightApp().run()
